@@ -4,6 +4,7 @@
  
 #include "server/connectionVTP.h"
 #include "server/livestreamer.h"
+#include "server/livefilter.h"
 #include "server/suspend.h"
 #include "setup.h"
 
@@ -37,16 +38,8 @@ enum eDumpModeStreamdev { dmsdAll, dmsdPresent, dmsdFollowing, dmsdAtTime, dmsdF
 class cLSTEHandler 
 {
 private:
-#if defined(USE_PARENTALRATING) || defined(PARENTALRATINGCONTENTVERSNUM)
-	enum eStates { Channel, Event, Title, Subtitle, Description, Vps, Content,
-	               EndEvent, EndChannel, EndEPG };
-#elif APIVERSNUM >= 10711
 	enum eStates { Channel, Event, Title, Subtitle, Description, Vps, Content, Rating,
 	               EndEvent, EndChannel, EndEPG };
-#else
-	enum eStates { Channel, Event, Title, Subtitle, Description, Vps, 
-	               EndEvent, EndChannel, EndEPG };
-#endif /* PARENTALRATING */
 	cConnectionVTP    *m_Client;
 	cSchedulesLock    *m_SchedulesLock;
 	const cSchedules  *m_Schedules;
@@ -285,11 +278,7 @@ bool cLSTEHandler::Next(bool &Last)
 		break;
 
 	case Vps:
-#if defined(USE_PARENTALRATING) || defined(PARENTALRATINGCONTENTVERSNUM) || APIVERSNUM >= 10711
 		m_State = Content;
-#else
-		m_State = EndEvent;
-#endif /* PARENTALRATING */
 		if (m_Event->Vps())
 #ifdef __FreeBSD__
 			return m_Client->Respond(-215, "V %d", m_Event->Vps());
@@ -300,18 +289,6 @@ bool cLSTEHandler::Next(bool &Last)
 			return Next(Last);
 		break;
 
-#if defined(USE_PARENTALRATING) || defined(PARENTALRATINGCONTENTVERSNUM)
-	case Content:
-		m_State = EndEvent;
-		if (!isempty(m_Event->GetContentsString())) {
-			char *copy = strdup(m_Event->GetContentsString());
-			cString cpy(copy, true);
-			strreplace(copy, '\n', '|');
-			return m_Client->Respond(-215, "G %i %i %s", m_Event->Contents() & 0xF0, m_Event->Contents() & 0x0F, copy);
-		} else
-			return Next(Last);
-		break;
-#elif APIVERSNUM >= 10711
 	case Content:
 		m_State = Rating;
 		if (!isempty(m_Event->ContentToString(m_Event->Contents()))) {
@@ -330,7 +307,6 @@ bool cLSTEHandler::Next(bool &Last)
 		else
 			return Next(Last);
 		break;
-#endif
 
 	case EndEvent:
 		if (m_Traverse) {
@@ -597,9 +573,7 @@ cLSTRHandler::cLSTRHandler(cConnectionVTP *Client, const char *Option):
 	if (*Option) {
 		if (isnumber(Option)) {
 			m_Recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
-#if defined(USE_STREAMDEVEXT) || APIVERSNUM >= 10705
 			m_Event = m_Recording->Info()->GetEvent();
-#endif
 			m_Info = true;
 			if (m_Recording == NULL) {
 				m_Errno = 501;
@@ -732,8 +706,24 @@ bool cLSTRHandler::Next(bool &Last)
 	return false;
 }
 
+class cStreamdevLoopPrevention {
+private:
+	bool Unlock;
+public:
+	cStreamdevLoopPrevention(const cChannel* Channel, bool LoopPrevention): Unlock(LoopPrevention) {
+		if (LoopPrevention)
+			cPluginManager::CallAllServices(LOOP_PREVENTION_SERVICE, (void *)Channel);
+	}
+	~cStreamdevLoopPrevention() {
+		if (Unlock)
+			cPluginManager::CallAllServices(LOOP_PREVENTION_SERVICE, NULL);
+	}
+};
+
 // --- cConnectionVTP ---------------------------------------------------------
 
+#define LOOP_PREVENTION(c) cStreamdevLoopPrevention LoopPrevention(c, m_LoopPrevention);
+
 cConnectionVTP::cConnectionVTP(void): 
 		cServerConnection("VTP"),
 		m_LiveSocket(NULL),
@@ -744,6 +734,7 @@ cConnectionVTP::cConnectionVTP(void):
 		m_DataSocket(NULL),
 		m_LastCommand(NULL),
 		m_StreamType(stTSPIDS),
+		m_ClientVersion(0),
 		m_FiltersSupport(false),
 		m_RecPlayer(NULL),
 		m_TuneChannel(NULL),
@@ -753,6 +744,10 @@ cConnectionVTP::cConnectionVTP(void):
 		m_LSTTHandler(NULL),
 		m_LSTRHandler(NULL)
 {
+	m_LoopPrevention = StreamdevServerSetup.LoopPrevention;
+	if (m_LoopPrevention)
+		// Loop prevention enabled - but is there anybody out there?
+		m_LoopPrevention = cPluginManager::CallFirstService(LOOP_PREVENTION_SERVICE);
 }
 
 cConnectionVTP::~cConnectionVTP() 
@@ -772,14 +767,15 @@ cConnectionVTP::~cConnectionVTP()
 	delete m_RecPlayer;
 }
 
-inline bool cConnectionVTP::Abort(void) const
+bool cConnectionVTP::Abort(void) const
 {
-	return m_LiveStreamer && m_LiveStreamer->Abort();
+	return !IsOpen() || (m_LiveStreamer && m_LiveStreamer->Abort()) ||
+		(!m_LiveStreamer && m_FilterStreamer && m_FilterStreamer->Abort());
 }
 
 void cConnectionVTP::Welcome(void) 
 {
-	Respond(220, "Welcome to Video Disk Recorder (VTP)");
+	Respond(220, "VTP/1.0 Welcome to Video Disk Recorder");
 }
 
 void cConnectionVTP::Reject(void)
@@ -833,12 +829,14 @@ bool cConnectionVTP::Command(char *Cmd)
 	}
 
 	if      (strcasecmp(Cmd, "CAPS") == 0) return CmdCAPS(param);
+	else if (strcasecmp(Cmd, "VERS") == 0) return CmdVERS(param);
 	else if (strcasecmp(Cmd, "PROV") == 0) return CmdPROV(param);
 	else if (strcasecmp(Cmd, "PORT") == 0) return CmdPORT(param);
 	else if (strcasecmp(Cmd, "READ") == 0) return CmdREAD(param);
 	else if (strcasecmp(Cmd, "TUNE") == 0) return CmdTUNE(param);
 	else if (strcasecmp(Cmd, "PLAY") == 0) return CmdPLAY(param);
 	else if (strcasecmp(Cmd, "PRIO") == 0) return CmdPRIO(param);
+	else if (strcasecmp(Cmd, "SGNL") == 0) return CmdSGNL(param);
 	else if (strcasecmp(Cmd, "ADDP") == 0) return CmdADDP(param);
 	else if (strcasecmp(Cmd, "DELP") == 0) return CmdDELP(param);
 	else if (strcasecmp(Cmd, "ADDF") == 0) return CmdADDF(param);
@@ -874,10 +872,12 @@ bool cConnectionVTP::CmdCAPS(char *Opts)
 		return Respond(220, "Capability \"%s\" accepted", Opts);
 	}
 
+#ifdef STREAMDEV_PS
 	if (strcasecmp(Opts, "PS") == 0) {
 		m_StreamType = stPS;
 		return Respond(220, "Capability \"%s\" accepted", Opts);
 	}
+#endif
 
 	if (strcasecmp(Opts, "PES") == 0) {
 		m_StreamType = stPES;
@@ -905,6 +905,17 @@ bool cConnectionVTP::CmdCAPS(char *Opts)
 	return Respond(561, "Capability \"%s\" not known", Opts);
 }
 
+bool cConnectionVTP::CmdVERS(char *Opts) 
+{
+	unsigned int major, minor;
+	if (sscanf(Opts, " %u.%u", &major, &minor) != 2)
+		return Respond(501, "Use: VERS version (with version in format major.minor)");
+
+	m_ClientVersion = major * 100 + minor;
+	m_FiltersSupport = true;
+	return Respond(220, "Protocol version %u.%u accepted", major, minor);
+}
+
 bool cConnectionVTP::CmdPROV(char *Opts) 
 {
 	const cChannel *chan;
@@ -919,15 +930,31 @@ bool cConnectionVTP::CmdPROV(char *Opts)
 	if ((chan = ChannelFromString(Opts)) == NULL)
 		return Respond(550, "Undefined channel \"%s\"", Opts);
 
+	// legacy clients use priority 0 even if live TV has priority
+	if (m_ClientVersion == 0 && prio == 0)
+		prio = StreamdevServerSetup.VTPPriority;
+
+	LOOP_PREVENTION(chan);
+
 	if (ProvidesChannel(chan, prio)) {
 		m_TuneChannel = chan;
 		m_TunePriority = prio;
 		return Respond(220, "Channel available");
 	}
-	else {
-		m_TuneChannel = NULL;
-		return Respond(560, "Channel not available");
+	// legacy clients didn't lower priority when switching channels,
+	// so get our own receiver temporarily out of the way
+	if (m_ClientVersion == 0) {
+		Detach();
+		bool provided = ProvidesChannel(chan, prio);
+		Attach();
+		if (provided) {
+			m_TuneChannel = chan;
+			m_TunePriority = prio;
+			return Respond(220, "Channel available");
+		}
 	}
+	m_TuneChannel = NULL;
+	return Respond(560, "Channel not available");
 }
 
 bool cConnectionVTP::CmdPORT(char *Opts) 
@@ -987,7 +1014,6 @@ bool cConnectionVTP::CmdPORT(char *Opts)
 		if(!m_FilterStreamer)
 			m_FilterStreamer = new cStreamdevFilterStreamer;
 		m_FilterStreamer->Start(m_FilterSocket);
-		m_FilterStreamer->Activate(true);
 
 		return Respond(220, "Port command ok, data connection opened");
 		break;
@@ -1086,15 +1112,16 @@ bool cConnectionVTP::CmdTUNE(char *Opts)
 	if ((chan = ChannelFromString(Opts)) == NULL)
 		return Respond(550, "Undefined channel \"%s\"", Opts);
 
-	if (chan != m_TuneChannel) {
-		esyslog("streamdev-server TUNE %s: Priority unknown - using 0", Opts);
-		prio = 0;
-	}
-	if ((dev = GetDevice(chan, prio)) == NULL)
-		return Respond(560, "Channel not available");
+	LOOP_PREVENTION(chan);
 
-	if (!dev->SwitchChannel(chan, false))
-		return Respond(560, "Channel not available");
+	if (chan != m_TuneChannel) {
+		isyslog("streamdev-server TUNE %s: Priority unknown - using 0", Opts);
+		prio = 0;
+		if (!ProvidesChannel(chan, prio))
+			return Respond(560, "Channel not available (ProvidesChannel)");
+	}
+	if ((dev = SwitchDevice(chan, prio)) == NULL)
+		return Respond(560, "Channel not available (SwitchDevice)");
 
 	delete m_LiveStreamer;
 	m_LiveStreamer = new cStreamdevLiveStreamer(prio, this);
@@ -1122,7 +1149,7 @@ bool cConnectionVTP::CmdPLAY(char *Opts)
 				if (m_RecPlayer) {
 					delete m_RecPlayer;
 				}
-				m_RecPlayer = new RecPlayer(recording);
+				m_RecPlayer = new RecPlayer(recording->FileName());
 				return Respond(220, "%llu (Bytes), %u (Frames)", (long long unsigned int) m_RecPlayer->getLengthBytes(), (unsigned int) m_RecPlayer->getLengthFrames());
 			}
 			else {
@@ -1154,6 +1181,18 @@ bool cConnectionVTP::CmdPRIO(char *Opts)
 	return Respond(550, "Priority not applicable");
 }
 
+bool cConnectionVTP::CmdSGNL(char *Opts) 
+{
+	if (m_LiveStreamer) {
+		int devnum = -1;
+		int signal = -1;
+		int quality = -1;
+		m_LiveStreamer->GetSignal(&devnum, &signal, &quality);
+		return Respond(220, "%d %d:%d", devnum, signal, quality);
+	}
+	return Respond(550, "Signal not applicable");
+}
+
 bool cConnectionVTP::CmdADDP(char *Opts) 
 {
 	int pid;
@@ -1242,7 +1281,8 @@ bool cConnectionVTP::CmdABRT(char *Opts)
 
 	switch (id) {
 	case siLive: 
-		DELETENULL(m_LiveStreamer); 
+		if (m_LiveStreamer)
+			m_LiveStreamer->Stop();
 		DELETENULL(m_LiveSocket);
 		break;
 	case siLiveFilter:
@@ -1273,10 +1313,9 @@ bool cConnectionVTP::CmdQUIT(void)
 
 bool cConnectionVTP::CmdSUSP(void) 
 {
-	if (StreamdevServerSetup.SuspendMode == smAlways || cSuspendCtl::IsActive())
+	if (cSuspendCtl::IsActive())
 		return Respond(220, "Server is suspended");
-	else if (StreamdevServerSetup.SuspendMode == smOffer 
-			&& StreamdevServerSetup.AllowSuspend) {
+	else if (StreamdevServerSetup.AllowSuspend) {
 		cControl::Launch(new cSuspendCtl);
 		cControl::Attach();
 		return Respond(220, "Server is suspended");
@@ -1337,7 +1376,11 @@ bool cConnectionVTP::CmdSTAT(const char *Option)
 	if (*Option) {
 		if (strcasecmp(Option, "DISK") == 0) {
 			int FreeMB, UsedMB;
+#if APIVERSNUM < 20102
 			int Percent = VideoDiskSpace(&FreeMB, &UsedMB);
+#else
+			int Percent = cVideoDirectory::VideoDiskSpace(&FreeMB, &UsedMB);
+#endif
 			Reply(250, "%dMB %dMB %d%%", FreeMB + UsedMB, FreeMB, Percent);
 		}
 		else if (strcasecmp(Option, "NAME") == 0) {
@@ -1622,7 +1665,7 @@ bool cConnectionVTP::CmdMOVC(const char *Option)
 								Reply(250,"Channel \"%d\" moved to \"%d\"", From, To);
 							}
 							else {
-								Reply(501, "Can't move channel to same postion");
+								Reply(501, "Can't move channel to same position");
 							}
 						}
 						else {
@@ -1734,7 +1777,7 @@ bool cConnectionVTP::CmdDELR(const char *Option)
 bool cConnectionVTP::CmdRENR(const char *Option)
 {
 	INIT_WRAPPER();
-#if defined(LIEMIKUUTIO)
+#if defined(LIEMIKUUTIO) && LIEMIKUUTIO < 132
 	bool recordings = Recordings.Update(true);
 	if (recordings) {
 		if (*Option) {
@@ -1742,17 +1785,9 @@ bool cConnectionVTP::CmdRENR(const char *Option)
 			int n = strtol(Option, &tail, 10);
 			cRecording *recording = Recordings.Get(n - 1);
 			if (recording && tail && tail != Option) {
-#if APIVERSNUM < 10704
-				int priority = recording->priority;
-				int lifetime = recording->lifetime;
-#endif
 				char *oldName = strdup(recording->Name());
 				tail = skipspace(tail);
-#if APIVERSNUM < 10704
-				if (recording->Rename(tail, &priority, &lifetime)) {
-#else
 				if (recording->Rename(tail)) {
-#endif
 					Reply(250, "Renamed \"%s\" to \"%s\"", oldName, recording->Name());
 					Recordings.ChangeState();
 					Recordings.TouchUpdate();
@@ -1774,7 +1809,7 @@ bool cConnectionVTP::CmdRENR(const char *Option)
 		Reply(550, "No recordings available");
 	}
 #else
-	Reply(501, "Rename not supported, please use LIEMIEXT");
+	Reply(501, "Rename not supported, please use LIEMIKUUTIO < 1.32");
 #endif /* LIEMIKUUTIO */
 	EXIT_WRAPPER();
 }
@@ -1783,11 +1818,8 @@ bool cConnectionVTP::Respond(int Code, const char *Message, ...)
 {
 	va_list ap;
 	va_start(ap, Message);
-#if APIVERSNUM < 10515
-	char *buffer;
-	if (vasprintf(&buffer, Message, ap) < 0)
-		buffer = strdup("???");
-	cString str(buffer, true);
+#if APIVERSNUM >= 10728
+	cString str = cString::vsprintf(Message, ap);
 #else
 	cString str = cString::sprintf(Message, ap);
 #endif
@@ -1802,3 +1834,14 @@ bool cConnectionVTP::Respond(int Code, const char *Message, ...)
 				Code < 0 ? -Code : Code,
 				Code < 0 ? '-' : ' ', *str);
 }
+
+cString cConnectionVTP::ToText() const
+{
+	cString str = cServerConnection::ToText();
+	if (m_LiveStreamer)
+		return cString::sprintf("%s\t%s", *str, *m_LiveStreamer->ToText());
+	else if (m_RecPlayer)
+		return cString::sprintf("%s\t%s", *str, m_RecPlayer->getCurrentRecording()->Name());
+	else
+		return str;
+}
