@@ -11,7 +11,6 @@
 #include <vdr/ringbuffer.h>
 
 #include "server/livestreamer.h"
-#include "server/livefilter.h"
 #include "common.h"
 
 using namespace Streamdev;
@@ -22,22 +21,23 @@ class cStreamdevLiveReceiver: public cReceiver {
 	friend class cStreamdevStreamer;
 
 private:
-	cStreamdevStreamer *m_Streamer;
+	cStreamdevLiveStreamer *m_Streamer;
 
 protected:
-	virtual void Activate(bool On);
 	virtual void Receive(uchar *Data, int Length);
 
 public:
-	cStreamdevLiveReceiver(cStreamdevStreamer *Streamer, tChannelID ChannelID, int Priority, const int *Pids);
+	cStreamdevLiveReceiver(cStreamdevLiveStreamer *Streamer, const cChannel *Channel, int Priority, const int *Pids);
 	virtual ~cStreamdevLiveReceiver();
 };
 
-cStreamdevLiveReceiver::cStreamdevLiveReceiver(cStreamdevStreamer *Streamer, tChannelID ChannelID, 
-                                               int Priority, const int *Pids):
-		cReceiver(ChannelID, Priority, 0, Pids),
+cStreamdevLiveReceiver::cStreamdevLiveReceiver(cStreamdevLiveStreamer *Streamer, const cChannel *Channel, int Priority, const int *Pids):
+		cReceiver(Channel, Priority),
 		m_Streamer(Streamer)
 {
+		// clears all PIDs but channel remains set
+		SetPids(NULL);
+		AddPids(Pids);
 }
 
 cStreamdevLiveReceiver::~cStreamdevLiveReceiver() 
@@ -47,15 +47,7 @@ cStreamdevLiveReceiver::~cStreamdevLiveReceiver()
 }
 
 void cStreamdevLiveReceiver::Receive(uchar *Data, int Length) {
-	int p = m_Streamer->Receive(Data, Length);
-	if (p != Length)
-		m_Streamer->ReportOverflow(Length - p);
-}
-
-inline void cStreamdevLiveReceiver::Activate(bool On) 
-{ 
-	Dprintf("LiveReceiver->Activate(%d)\n", On);
-	m_Streamer->Activate(On); 
+	m_Streamer->Receive(Data, Length);
 }
 
 // --- cStreamdevPatFilter ----------------------------------------------------
@@ -320,9 +312,9 @@ void cStreamdevPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, i
 		pids[npids++] = pmtPid;
 #if 0
 		pids[npids++] = 0x10;  // pid 0x10, tid 0x40: NIT 
+#endif
 		pids[npids++] = 0x11;  // pid 0x11, tid 0x42: SDT 
 		pids[npids++] = 0x14;  // pid 0x14, tid 0x70: TDT 
-#endif
 		pids[npids++] = 0x12;  // pid 0x12, tid 0x4E...0x6F: EIT 
 		for (SI::Loop::Iterator it; pmt.streamLoop.getNext(stream, it); )
 			if (0 != (pids[npids] = GetPid(stream)) && npids < MAXRECEIVEPIDS)
@@ -346,18 +338,18 @@ cStreamdevLiveStreamer::cStreamdevLiveStreamer(int Priority, const cServerConnec
 		m_PatFilter(NULL),
 		m_Remux(NULL)
 {
+		m_ReceiveBuffer = new cStreamdevBuffer(LIVEBUFSIZE, TS_SIZE *2, true, "streamdev-livestreamer"),
+		m_ReceiveBuffer->SetTimeouts(0, 100);
 }
 
 cStreamdevLiveStreamer::~cStreamdevLiveStreamer() 
 {
 	Dprintf("Desctructing Live streamer\n");
 	Stop();
-	if(m_PatFilter) {
-		Detach();
-		DELETENULL(m_PatFilter);
-	}
+	DELETENULL(m_PatFilter);
 	DELETENULL(m_Receiver);
 	delete m_Remux;
+	delete m_ReceiveBuffer;
 }
 
 bool cStreamdevLiveStreamer::HasPid(int Pid) 
@@ -440,13 +432,36 @@ void cStreamdevLiveStreamer::SetPriority(int Priority)
 	StartReceiver();
 }
 
+void cStreamdevLiveStreamer::GetSignal(int *DevNum, int *Strength, int *Quality) const
+{
+	if (m_Device) {
+		*DevNum = m_Device->DeviceNumber() + 1;
+		*Strength = m_Device->SignalStrength();
+		*Quality = m_Device->SignalQuality();
+	}
+}
+
+cString cStreamdevLiveStreamer::ToText() const
+{
+	if (m_Device && m_Channel) {
+		return cString::sprintf("DVB%-2d %3d %s", m_Device->DeviceNumber() + 1, m_Channel->Number(), m_Channel->Name());
+	}
+        return cString("");
+}
+
+bool cStreamdevLiveStreamer::IsReceiving(void) const
+{
+	cThreadLock ThreadLock(m_Device);
+	return m_Receiver && m_Receiver->IsAttached();
+}
+
 void cStreamdevLiveStreamer::StartReceiver(void)
 {
 	if (m_NumPids > 0) {
 		Dprintf("Creating Receiver to respect changed pids\n");
 		cReceiver *current = m_Receiver;
-		m_Receiver = new cStreamdevLiveReceiver(this, m_Channel->GetChannelID(), m_Priority, m_Pids);
 		cThreadLock ThreadLock(m_Device);
+		m_Receiver = new cStreamdevLiveReceiver(this, m_Channel, m_Priority, m_Pids);
 		if (IsRunning())
 			Attach();
 		delete current;
@@ -481,9 +496,11 @@ bool cStreamdevLiveStreamer::SetChannel(const cChannel *Channel, eStreamType Str
 		m_Remux = new cTS2PESRemux(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids());
 		return SetPids(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids());
 
+#ifdef STREAMDEV_PS
 	case stPS:  
 		m_Remux = new cTS2PSRemux(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids());
 		return SetPids(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids());
+#endif
 
 	case stEXT:
 		m_Remux = new cExternRemux(Connection(), m_Channel, Apids, Dpids);
@@ -510,6 +527,13 @@ bool cStreamdevLiveStreamer::SetChannel(const cChannel *Channel, eStreamType Str
 	}
 }
 
+void cStreamdevLiveStreamer::Receive(uchar *Data, int Length)
+{
+	int p = m_ReceiveBuffer->PutTS(Data, Length);
+	if (p != Length)
+		m_ReceiveBuffer->ReportOverflow(Length - p);
+}
+
 int cStreamdevLiveStreamer::Put(const uchar *Data, int Count) 
 {
 	// insert si data
@@ -589,105 +613,3 @@ std::string cStreamdevLiveStreamer::Report(void)
 	result += "\n";
 	return result;
 }
-
-// --- cStreamdevFilterStreamer -------------------------------------------------
-
-cStreamdevFilterStreamer::cStreamdevFilterStreamer():
-		cStreamdevStreamer("streamdev-filterstreaming"),
-		m_Device(NULL),
-		m_Filter(NULL)/*,
-		m_Channel(NULL)*/
-{
-}
-
-cStreamdevFilterStreamer::~cStreamdevFilterStreamer() 
-{
-	Dprintf("Desctructing Filter streamer\n");
-	Detach();
-	m_Device = NULL;
-	DELETENULL(m_Filter);
-	Stop();
-}
-
-void cStreamdevFilterStreamer::Attach(void) 
-{ 
-	Dprintf("cStreamdevFilterStreamer::Attach()\n");
-	LOCK_THREAD;
-	if(m_Device && m_Filter)
-		m_Device->AttachFilter(m_Filter);
-}
-
-void cStreamdevFilterStreamer::Detach(void) 
-{ 
-	Dprintf("cStreamdevFilterStreamer::Detach()\n");
-	LOCK_THREAD;
-	if(m_Device && m_Filter)
-		m_Device->Detach(m_Filter); 
-}
-
-#if 0
-void cStreamdevFilterStreamer::SetChannel(const cChannel *Channel)
-{
-	LOCK_THREAD;
-	Dprintf("cStreamdevFilterStreamer::SetChannel(%s : %s)", Channel?Channel->Name():"<null>",
-		Channel ? *Channel->GetChannelID().ToString() : "");
-	m_Channel = Channel;
-}
-#endif
-
-void cStreamdevFilterStreamer::SetDevice(cDevice *Device)
-{
-	Dprintf("cStreamdevFilterStreamer::SetDevice()\n");
-	LOCK_THREAD;
-	Detach();
-	m_Device = Device;
-	//m_Channel = NULL;
-	Attach();
-}
-
-bool cStreamdevFilterStreamer::SetFilter(u_short Pid, u_char Tid, u_char Mask, bool On) 
-{	
-	Dprintf("cStreamdevFilterStreamer::SetFilter(%u,0x%x,0x%x,%s)\n", Pid, Tid, Mask, On?"On":"Off");
-
-	if(!m_Device)
-		return false;
-
-	if (On) {
-		if (m_Filter == NULL) {
-			m_Filter = new cStreamdevLiveFilter(this);
-			Dprintf("attaching filter to device\n");
-			Attach();
-		}
-		m_Filter->Set(Pid, Tid, Mask);
-	} else if (m_Filter != NULL) 
-		m_Filter->Del(Pid, Tid, Mask);
-
-	return true;
-}
-
-#if 0
-void cStreamdevFilterStreamer::ChannelSwitch(const cDevice *Device, int ChannelNumber) {
-	LOCK_THREAD;
-	if(Device == m_Device) {
-		if(ChannelNumber > 0) {
-			cChannel *ch = Channels.GetByNumber(ChannelNumber);
-			if(ch != NULL) {
-				if(m_Filter != NULL &&
-						m_Channel != NULL &&
-						(! TRANSPONDER(ch, m_Channel))) {
-
-					isyslog("***** LiveFilterStreamer: transponder changed ! %s",
-						*ch->GetChannelID().ToString());
-
-					uchar buffer[TS_SIZE] = {TS_SYNC_BYTE, 0xff, 0xff, 0xff, 0x7f, 0};
-					strcpy((char*)(buffer + 5), ch->GetChannelID().ToString());
-					int p = Put(buffer, TS_SIZE);
-					if (p != TS_SIZE)
-						ReportOverflow(TS_SIZE - p);
-				}
-				m_Channel = ch;
-			}
-		}
-	}
-}
-#endif
