@@ -12,6 +12,7 @@
 class cPluginStreamdevServer : public cPlugin {
 private:
 	static const char *DESCRIPTION;
+	bool m_Suspend;
 
 public:
 	cPluginStreamdevServer(void);
@@ -26,6 +27,7 @@ public:
 	virtual cString Active(void);
 	virtual const char *MainMenuEntry(void);
 	virtual cOsdObject *MainMenuAction(void);
+	virtual void MainThreadHook(void);
 	virtual cMenuSetupPage *SetupMenu(void);
 	virtual bool SetupParse(const char *Name, const char *Value);
 };
